<?php


namespace Schaffen\ServiceOrder\ServiceOrder\Repositories;

use Exception;
use Freelabois\LaravelQuickstart\Extendables\Repository;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\DB;
use Schaffen\ServiceOrder\ServiceOrder\Models\ServiceOrder;
use stdClass;

class ServiceOrderRepository extends Repository
{
    protected $model = ServiceOrder::class;

    protected $searchableFields = [
        ["field" => "os_codigo"],
        ["field" => "os_end_cep"],
        [
            "field" => "os_tipo",
            'operator' => "ilike"
        ]
    ];

    public function dd(array $filters = [], array $with = [], $pagination = 45)
    {
        $this->applyFilters($filters);
        $query = $this->newQuery();
        $query->with($with);
        if ($pagination === "false" || $pagination === false) {
            $pagination = 9223372036854775807;
        }
        if (!empty($this->filters)) {
            dump($this->filters);
            $this->applyCustomFilters();
            $this->injectFiltersOnQuery();
        }
        $this->group();
        $this->order();
        $query->dd();
    }

    public function injectFiltersOnQuery()
    {
        $os_recurso = null;
        $not_final = null;
        $not_final_or_executing = null;
        $has_scheduling = null;
        $not_executed = null;
        $schedule_for_today_or_older = null;

        extract($this->filters);

        $this->query->when($schedule_for_today_or_older, function (Builder $query){
            return $query->where('os_data_agendamento', '<', now()->endOfDay());
        });

        $this->query->when($os_recurso, function ($query) use ($os_recurso) {
            $query->crossJoin(DB::raw("lateral json_array_elements(os.os_recursos) as recurso(elem)"));
            return $query->whereRaw(" elem->'recurso'->>'pes_codigo' = ?", ["$os_recurso"]);
        });

        $this->query->when($not_final || $not_executed || $not_final_or_executing, function ($query) use ($not_final, $not_executed, $not_final_or_executing){
            $query->whereHas('status', function ($query) use ($not_final, $not_executed, $not_final_or_executing){
                $query->when($not_executed, function($query){
                    return $query->where('sto_executado', '=', 0);
                });
                $query->when($not_final, function($query){
                    return $query->whereHas('nextStatus');
                });
                $query->when($not_final_or_executing, function(Builder $query){
                    return $query->where(function ($query){
                        $query->where("sto_codigo", ServiceOrder::STATUS_EM_EXECUCAO);
                        return $query->orWhereHas('nextStatus');
                    });
                });
                return $query;
            });
        });

        $this->query->when($has_scheduling, function ($query) {
            return $query->where('os_data_agendamento', '!=', null);
        });

        parent::injectFiltersOnQuery();
    }

    public function getItensNaoFaturados($osCodigo, $servicos = 0)
    {
        $itensOs = OsProdutosServicos::where("osps_fk_os_codigo", $osCodigo)
            ->whereNull("osps_fk_fat_codigo")
            ->where("osps_servico", $servicos)
            ->orderBy("osps_numero_sequencia")->get();

        $arrayTipoVenda = [
            ["codigo" => Venda::TIPO_PEDIDO, "descricao" => "Venda"],
            ["codigo" => Venda::TIPO_NFCE, "descricao" => "NFCe"],
        ];


        //verifica se a empresa tem acesso a emissão de NFe
        if (Modulo::hasNFe()) {
            $arrayTipoVenda[] = ["codigo" => Venda::TIPO_NFE, "descricao" => "NFe"];
        }

        $dados = [
            "bandeiras" => FormaPagamento::getBandeiras(),
            "formas_pagamento" => FormaPagamento::select("fop_codigo", "fop_descricao", "fop_prazo", "fop_bandeira", "fop_numero_maximo_parcelas")->orderBy("fop_descricao")->get(),
            "tipo_venda" => $arrayTipoVenda,
            "faturamento_venda" => false, //força sempre false pois o faturamento da OS é gerado em outro momento
            "fechamentoCaixa" => Modulo::hasFechamentoCaixa()
        ];

        return Response::success("", ["itens" => $itensOs, "dados" => $dados]);
    }

    public function getOsGerarNotaFiscal($tipoNota, $filtro)
    {
        $filtroObject = json_decode($filtro);
        if (json_last_error() !== JSON_ERROR_NONE) {
            return Response::faillure('Objeto json de filtro é inválido: ' . json_last_error_msg(), null, 400);
        }
//retorna os faturamentos q não possuem notas com o STATUS = 100 -> autorizadas
        switch ($tipoNota) {
            case 'NFE':
            case 'NFCE':
                $sqlNotExists = "fnf_fk_fat_codigo = fat_codigo
                             and exists (select 1
                                           from sefaz.documento
                                          where (doc_status not in ('135', '102') or doc_status is null)
                                            and doc_fk_nof_codigo = fnf_fk_nof_codigo)";
                $filtroObject[] = SCHModel::buildWhereNotExistsObject("os.faturamento_nota_fiscal", $sqlNotExists);
                break;
            case 'NFSE':
                $sqlNotExists = "fnf_fk_fat_codigo = fat_codigo
                             and exists (select 1
                                           from fiscal.nota_fiscal_servico_documento
                                          where (nfsd_status not in  ('135', '102') or nfsd_status is null)
                                            and nfsd_fk_nfs_codigo = fnf_fk_nfs_codigo)";
                $filtroObject[] = SCHModel::buildWhereNotExistsObject("os.faturamento_nota_fiscal", $sqlNotExists);
                break;
            default:
                return Response::faillure("Tipo de nota fiscal solicitada é inválida.");
        }
        $filtroObject[] = SCHModel::buildFilterObject("fat_data_estorno", SCHModel::OPERATOR_IS, '');
        return $this->searchData($filtroObject, null, Faturamento::class, ["cliente", "filial"]);
    }

    public function verificaFaturamento($codigoOs)
    {
        $permiteFaturamentoMVF = false;
        $permiteGerarVendaItens = false;
        $permiteGerarNFSe = false;

        $os = Os::find($codigoOs);

        if (!$os) throw new SCHException("OS código {$codigoOs} não encontrada.");

        $faturamento = new FaturamentoOs();
        $params['os_numero_tipo'] = $os['os_numero_tipo'];
        $faturamento->getOsFaturamento($params);

        $faturamento = json_decode(json_encode($faturamento), true);
        $faturamento = $faturamento['result'][0];

        $valorRestante = number_format(floatval(number_format($faturamento['total'], 2, '.', '')) - floatval(number_format($faturamento['total_mvf'], 2, '.', '')), 2, '.', '');

        if ($faturamento['sto_permite_faturamento'] == 1 && $valorRestante <= 0) $permiteFaturamentoMVF = true;

        if ($faturamento['possui_itens_faturamento_servico_renovacao'] > 1 && $faturamento['sto_permite_faturamento'] == 1) $permiteFaturamentoMVF = true;

        if ($faturamento['possui_itens_faturamento'] == 0) $permiteGerarVendaItens = true;

        if ($faturamento['possui_itens_faturamento_servico'] == 0) $permiteGerarNFSe = true;

        if ($permiteFaturamentoMVF && $permiteGerarVendaItens && $permiteGerarNFSe) {
            try {
                DB::beginTransaction();
                $param1209 = Parametro::getDadosParametro(1209)->par_valor;

                if ($param1209 == 1) {
                    $os->os_fk_sto_codigo = -1;
                    $os->save();
                }


                DB::commit();
            } catch (Exception $e) {
                DB::rollback();
            }
        }
    }

    public function verificaPessoaMvfAtrasada($cod)
    {
        if (!isset($cod) || empty($cod)) {
            return Response::error("Usuario invalido");
        }
        $pessoa = Pessoa::find($cod);
        if (!isset($pessoa->pes_dias_bloqueio_os) || empty($pessoa->pes_dias_bloqueio_os)) {
            return Response::success('', true);
        }
        $filtro[0] = new stdClass();
        $filtro[0]->column = 'mvf_fk_pes_codigo';
        $filtro[0]->operator = '=';
        $filtro[0]->value = $cod;
        $filtro[1] = new stdClass();
        $filtro[1]->column = 'mvf_sentido';
        $filtro[1]->operator = 'LIKE';
        $filtro[1]->value = 'C';
        $parcelas = Parcela::getBlocoRecibo($filtro);
        foreach ($parcelas as $parcela) {
            if ($parcela->par_status_desc == "Vencido") {
                $now = Carbon::now();
                if (!Carbon::createFromFormat('d/m/Y', $parcela->par_data_vencimento_original)->addDays($pessoa->pes_dias_bloqueio_os)->gte($now)) {
                    return Response::faillure('Cliente possui pendência. Contate o departamento financeiro para mais informações e/ou liberação', false);
                }
            }
        }
        return Response::success('', true);
    }

    public function verificaPessoaMvfPendente($cod)
    {
        if (!isset($cod) || empty($cod)) {
            return Response::error("Usuario invalido");
        }
        $pessoa = Pessoa::find($cod);

        $filtro[0] = new stdClass();
        $filtro[0]->column = 'mvf_fk_pes_codigo';
        $filtro[0]->operator = '=';
        $filtro[0]->value = $cod;
        $filtro[1] = new stdClass();
        $filtro[1]->column = 'mvf_sentido';
        $filtro[1]->operator = 'LIKE';
        $filtro[1]->value = 'C';
        $parcelas = Parcela::getBlocoRecibo($filtro);
        foreach ($parcelas as $parcela) {
            if ($parcela->par_status_desc == "Vencido") {
                return Response::faillure('Cliente selecionado possui pendência financeira.', false);
            }
        }
        return Response::success('', true);
    }

    public function verificaPessoaBloqueioFinanceiro($cod)
    {
        if (!isset($cod) || empty($cod)) {
            return Response::error("Usuario invalido");
        }
        $pessoa = Pessoa::find($cod);

        if ($pessoa->pes_limite_bloqueio) {
            return Response::faillure('Cliente está bloqueado. Motivo: "' . $pessoa->pes_limite_observacao . '". Contate o departamento financeiro para mais informações e/ou liberação', false);
        }

        return Response::success('', true);
    }

    public function searchImpressaoMatricial(Request $request)
    {
        $paramsRequest = [];

        if ($request->os_codigo_or_number != null) {
            $paramsRequest['os_codigo_or_number'] = $request->os_codigo_or_number;
        }

        if ($request->os_fk_pes_codigo != null) {
            $paramsRequest['os_fk_pes_codigo'] = $request->os_fk_pes_codigo;
        }

        if ($request->os_fk_sto_codigo != null) {
            $paramsRequest['os_fk_sto_codigo'] = $request->os_fk_sto_codigo;
        }

        $ordensServico = Os::consultaByParams($paramsRequest);

        if (count($ordensServico) > 0) {
            foreach ($ordensServico as $os) {
                /* Preenche os dados da Filial */
                $os->filial = $filial = Filial::with('pessoa')->find($os->os_fk_fil_codigo);
                /* Preenche dados dos produtos */
                $osProdutos = new OsProdutosServicos();
                $os->produtos = $osProdutos->consultaByOs($os->os_codigo);
            }
        }

        return Response::success('', $ordensServico);
    }

    public function searchImpressaoMatricialAfras(Request $request)
    {
        $paramsRequest = [];

        if ($request->os_codigo_or_number != null) {
            $paramsRequest['os_codigo_or_number'] = $request->os_codigo_or_number;
        }

        if ($request->os_fk_pes_codigo != null) {
            $paramsRequest['os_fk_pes_codigo'] = $request->os_fk_pes_codigo;
        }

        if ($request->os_fk_sto_codigo != null) {
            $paramsRequest['os_fk_sto_codigo'] = $request->os_fk_sto_codigo;
        }

        $ordensServico = Os::consultaByParamsAfras($paramsRequest);

        if (count($ordensServico) > 0) {
            foreach ($ordensServico as $os) {

                foreach (json_decode($os->os_campos) as $campo) {
                    switch ($campo->id) {
                        case -1:
                            $os->adicional = $campo->valor;
                            break;
                        case -10:
                            $os->valor_hr_tecnica = $campo->valor;
                            break;
                        case -11:
                            $os->adicional1 = $campo->valor;
                            break;
                        case -12:
                            $os->adicional2 = $campo->valor;
                            break;
                        case -13:
                            $os->sistema_associado = $campo->valor;
                            break;
                        default:
                            break;
                    }
                }


                /* Preenche os dados da Filial */
                $os->filial = $filial = Filial::with('pessoa')->find($os->os_fk_fil_codigo);
                /* Preenche dados dos produtos */
                $osProdutos = new OsProdutosServicos();
                $os->produtos = $osProdutos->consultaByOs($os->os_codigo);
            }
        }

        return Response::success('', $ordensServico);
    }

    public function getDadosNFe($os_codigo, $fat_codigo)
    {
        $dadosOs = Os::where('os_codigo', '=', $os_codigo)->first();

        if (!empty($fat_codigo) && $fat_codigo != 'null') {
            /* Busca as movimentações do faturamento */
            $fatMvf = FaturamentoMvf::where('fam_fk_fat_codigo', '=', $fat_codigo)
                ->orderBy("fam_fk_mvf_codigo")
                ->get();
            /* Busca as parcelas referente a MVF gerada no faturamento para a OS */
            if (count($fatMvf) > 0) {
                $parcelasOs = [];
                $countParcOs = 0;
                foreach ($fatMvf as $key => $fatMvf) {
                    $parc = Parcela::where('par_fk_mvf_codigo', '=', $fatMvf->fam_fk_mvf_codigo)->first();
                    $parcelasOs[$countParcOs] = $parc;
                    $countParcOs++;
                }
            }

            $dadosOs->numero_parcelas = count($parcelasOs);
            $dadosOs->parcelas = $parcelasOs;
        }

        //obtém os itens da composição
        foreach ($dadosOs->composicoes as $key => $composicao) {
            if (!empty($composicao->osps_fk_cc_codigo_deb)) {
                $itensComposicao = EstoqueItemComposicao::where("eic_fk_cc_codigo_origem", "=", $composicao->osps_fk_cc_codigo_deb)->get();
                $itens = ProdutoComposicaoController::converteResultsetComposicao2RateioComposicao($itensComposicao);
            } else {
                $itens = ProdutoComposicaoController::getProdutosRateioComposicao(
                    $composicao->produto->pro_codigo, $composicao->osps_valor_unitario, $composicao->osps_quantidade, $composicao->osps_desconto, ProdutoComposicaoController::TIPO_DESCONTO_VALOR, true, ProdutoComposicaoController::RETORNA_PRODUTOS_E_SERVICOS
                );
            }
            $dadosOs->composicoes[$key]->itens = $itens;
        }

        return Response::success('', $dadosOs);
    }

    /**
     * Retorna objeto para emissão da NFSe - o objeto retornado será utilizado
     * na interface de emissão de NFSe
     *
     * @param type $osCodigo
     * @return type
     * @throws SCHException
     */
    public static function getDadosNFSe($osCodigo)
    {

        try {
            $os = Os::find($osCodigo);
            if (!$os) {
                throw new SCHException("A OS/Orçamento código {$osCodigo} não foi encontrada.");
            }

            $nfs = [
                "os_codigo" => $os->os_codigo,
                "nfs_data_emissao" => date('d/m/Y'),
                "nfs_fk_fil_codigo" => $os->os_fk_fil_codigo,
                "nfs_discriminacao_servico" => $os->os_discriminacao_servico,
                "tomador" => [
                    "pes_codigo" => $os->cliente->pes_codigo,
                    "pes_nome" => $os->cliente->pes_nome,
                    "pee_cep" => $os->os_end_cep,
                    "pee_logradouro" => $os->os_end_logradouro,
                    "pee_numero" => $os->os_end_numero,
                    "pee_bairro" => $os->os_end_bairro,
                    "pee_uf" => $os->os_end_uf,
                    "pee_cidade" => $os->os_end_cidade,
                ],
                "nfs_fk_pes_codigo_tomador" => $os->os_fk_pes_codigo,
                "nfs_tomador_end_cep" => $os->os_end_cep,
                "nfs_tomador_end_logradouro" => $os->os_end_logradouro,
                "nfs_tomador_end_numero" => $os->os_end_numero,
                "nfs_tomador_end_bairro" => $os->os_end_bairro,
                "nfs_tomador_end_uf" => $os->os_end_uf,
                "nfs_tomador_end_municipio" => $os->os_end_cidade,
                "itens" => [],
            ];
            //retorna os serviços
            $osServicos = OsProdutosServicos::where("osps_fk_os_codigo", $osCodigo)->where("osps_servico", 1)
                ->whereNull("osps_fk_fat_codigo")
                ->get();
            foreach ($osServicos as $servico) {

                $duplicado = -1;
                $servico->produto->nome_completo_nfse = $servico->produto->nome_completo;
                foreach ($nfs['itens'] as $key => $itens) {
                    if ($itens['nfsi_codigo_servico_pref'] == $servico->produto->pro_codigo_servico_municipio) {
                        $duplicado = $key;
                        break;
                    }
                }
                if ($duplicado != -1 && Parametro::getDadosParametro(702)->par_valor == 1) {
                    $nfs['itens'][$duplicado]["nfsi_valor_unitario"] = $nfs['itens'][$duplicado]["nfsi_quantidade"] * $nfs['itens'][$duplicado]["nfsi_valor_unitario"];
                    $nfs['itens'][$duplicado]["nfsi_quantidade"] = 1;
                    $nfs['itens'][$duplicado]["total"] = ($nfs['itens'][$duplicado]["nfsi_quantidade"] * $nfs['itens'][$duplicado]["nfsi_valor_unitario"]) - $nfs['itens'][$duplicado]["nfsi_valor_desconto"];
                    $nfs['itens'][$duplicado]["nfsi_valor_desconto"] += $servico->osps_desconto;
                    $nfs['itens'][$duplicado]["nfsi_valor_unitario"] += $servico->osps_valor_unitario * $servico->osps_quantidade;
                    $nfs['itens'][$duplicado]["total"] += ($servico->osps_valor_unitario * $servico->osps_quantidade) - $servico->osps_desconto;
                } else {
                    $nfs['itens'][] = [
                        "nfsi_fk_pve_codigo" => $servico->osps_fk_pve_codigo,
                        "nfsi_descricao_servico" => $servico->produto->nome_completo,
                        "nfsi_codigo_servico_pref" => $servico->produto->pro_codigo_servico_municipio,
                        "nfsi_cnae" => $servico->produto->pro_cnae,
                        "nfsi_quantidade" => $servico->osps_quantidade,
                        "nfsi_valor_unitario" => $servico->osps_valor_unitario,
                        "nfsi_valor_desconto" => $servico->osps_desconto,
                        "nfsi_tributavel" => 1,
                        "total" => ($servico->osps_quantidade * $servico->osps_valor_unitario) - $servico->osps_desconto,
                        "servico" => $servico->produto
                    ];
                }
            }

            //verifica se possui itens de locação
            if ($os->locacoes && count($os->locacoes) > 0) {
                // verifica se o parametro do serviço de locações está definido
                $param = Parametro::getDadosParametro(Parametro::SERVICO_LOCACAO_NFS);
                if (!$param || empty($param->par_valor)) {
                    throw new SCHException("Não foi definido o código do serviço para emissão da NFS dos itens de locação (parâmetro " . Parametro::SERVICO_LOCACAO_NFS . ")");
                }

                // busca pelo produto
                $produtoLocacao = ProdutoVariedadeEmbalagem::getDadosEmbalagem($param->par_valor);
                if (!$produtoLocacao || count($produtoLocacao) == 0) {
                    throw new SCHException("Não foi definido um código de serviço válido no parâmetro " . Parametro::SERVICO_LOCACAO_NFS . ".");
                }

                $produtoLocacao = $produtoLocacao[0];
                $valorTotalLocacao = 0;
                // obtém o total da locação
                foreach ($os->locacoes as $locacao) {
                    if (!empty($locacao->olo_data_devolucao) && empty($locacao->olo_fk_nfs_codigo)) {
                        $valorMulta = empty($locacao->olo_valor_multa) || empty($locacao->olo_incluir_multa_nfs) ?
                            0 : $locacao->olo_valor_multa;
                        $valorTotalLocacao += $valorMulta + $locacao->olo_total_locacao;
                    }
                }
                if ($valorTotalLocacao > 0) {
                    $produtoLocacao->nome_completo_nfse = $produtoLocacao->pro_descricao . ' - ' . $produtoLocacao->prv_descricao;
                    $nfs['itens'][] = [
                        "nfsi_fk_pve_codigo" => $produtoLocacao->pve_codigo,
                        "nfsi_descricao_servico" => $produtoLocacao->pro_descricao . ' - ' . $produtoLocacao->prv_descricao,
                        "nfsi_codigo_servico_pref" => $produtoLocacao->pro_codigo_servico_municipio,
                        "nfsi_cnae" => $produtoLocacao->pro_cnae,
                        "nfsi_quantidade" => 1,
                        "nfsi_valor_unitario" => $valorTotalLocacao,
                        "nfsi_valor_desconto" => 0,
                        "nfsi_tributavel" => $produtoLocacao->pro_servico_isento_iss,
                        "total" => $valorTotalLocacao,
                        "servico" => $produtoLocacao
                    ];
                }
            }

            if (count($nfs['itens']) == 0) {
                throw new SCHException("Não existem serviços para serem faturados a partir da OS/Orçamento {$osCodigo}.");
            }

            return Response::success('', $nfs);
        } catch (SCHException $e) {
            return Response::faillure($e->getMessage(), $e->getTrace());
        } catch (Exception $e) {
            return Response::error($e->getMessage(), $e);
        }
    }

    public static function consultaFaturamentoMvfOs($osCodigo)
    {
        return Response::success('', Os::consultaFaturamentoMvfOs($osCodigo));
    }

    public static function consultaFaturamentoBoleto($fatCodigo)
    {
        return Response::success('', Os::consultaFaturamentoBoleto($fatCodigo));
    }

    public static function consultaFaturamentoNotaFiscal($osCodigo)
    {
        $nota = FaturamentoNotaFiscal::where('fnf_fk_os_codigo', $osCodigo)->first();
        $notaFiscal = (object)Array();
        if (!empty($nota)) {
            $notaFiscal->nof_codigo = $nota->fnf_fk_nof_codigo;
            $notaFiscal->doc_status = 0;
            $notaFiscal->doh_motivo = '';
            $documento = Documento::where("doc_fk_nof_codigo", $nota->fnf_fk_nof_codigo)->first();
            if (!empty($documento)) {
                $notaFiscal->doc_status = $documento->doc_status;
                $documentoHistorico = DocumentoHistorico::where("doh_fk_doc_codigo", $documento->doc_codigo)->orderBy("doh_codigo", "desc")->first();
                $notaFiscal->doc_status_desc = $documentoHistorico->doh_motivo;
            }
        }
        return Response::success('', $notaFiscal);
    }

    /**
     * Retorna os dados de todas as parcelas vinculadas a uma OS.
     * As MVFs são geradas a partir do faturamento.
     *
     * @param type $osCodigo
     * @return type
     */
    public function getParcelasOs($osCodigo)
    {
        //obtém os faturamentos
        $osFaturamento = FaturamentoOs::where("fao_fk_os_codigo", $osCodigo)->get();
        $arrayMvfs = [];
        // obtém as movimentações financeiras
        foreach ($osFaturamento as $fatOs) {
            $faturamentoMvfs = FaturamentoMvf::where("fam_fk_fat_codigo", $fatOs->fao_fk_fat_codigo)->get();
            foreach ($faturamentoMvfs as $faturamentoMvf) {
                $arrayMvfs[] = $faturamentoMvf->fam_fk_mvf_codigo;
            }
        }

        if (count($arrayMvfs) == 0) {
            return Response::success("", []);
        }

        //consulta todas as parcelas dos faturamentos
        $parcelas = Parcela::getParcelas([SCHModel::buildFilterINObject("par_fk_mvf_codigo", $arrayMvfs)]);

        return Response::success("", $parcelas);
    }

    /**
     * Retorna as solicitações de compra vinculadas a uma OS
     * @param type $osCodigo
     * @return type
     */
    public function getSolicitacaoCompra($osCodigo)
    {
        $rs = SolicitacaoCompra::getVinculoModulos(ModuloSolicitacaoCompra::MOD_ORDEM_SERVICO, $osCodigo);
        return Response::success("", $rs);
    }

    /**
     * Retorna os dados de todas as vendas vinculadas a uma OS.
     * As vendas são geradas a partir do faturamento.
     *
     * @param type $osCodigo
     * @return type
     */
    public function getVendasOs($osCodigo)
    {
        //obtém os faturamentos
        $osFaturamento = FaturamentoOs::where("fao_fk_os_codigo", $osCodigo)->get();
        $arrayVendas = [];
        // obtém as movimentações financeiras
        foreach ($osFaturamento as $fatOs) {
            $faturamentoMvfs = FaturamentoVenda::where("fav_fk_fat_codigo", $fatOs->fao_fk_fat_codigo)->get();
            foreach ($faturamentoMvfs as $faturamentoMvf) {
                $arrayVendas[] = $faturamentoMvf->fav_fk_ven_codigo;
            }
        }

        if (count($arrayVendas) == 0) {
            return Response::success("", []);
        }

        //consulta todas as parcelas dos faturamentos
        $vendas = Venda::getVendas([SCHModel::buildFilterINObject("ven_codigo", $arrayVendas)]);

        return Response::success("", $vendas);
    }

    /**
     * Retorna as notas fiscais de serviços e itens vinculadas à ordem de serviço
     *
     * @param type $osCodigo código da ordem de serviço
     * @return type
     */
    public function getNotasFiscaisOs($osCodigo)
    {

        $notas = [
            "servicos" => Os::getDadosNotasServicos($osCodigo),
            "itens" => Os::getDadosNotasItens($osCodigo)
        ];

        return Response::success("", $notas);
    }

    /**
     * Pesquisa os faturamentos realizados
     *
     * @param type $filtroObject
     * @return type
     */
    public function getFaturamentosRealizados($filtroObject)
    {
        if (!is_array($filtroObject)) {
            $filtroObject = json_decode($filtroObject);
            if (json_last_error() !== JSON_ERROR_NONE) {
                return Response::faillure('Objeto json de filtro é inválido: ' . json_last_error_msg(), null, 400);
            }
        }
        return Response::success("", Faturamento::getFaturamentos($filtroObject));
    }

    /**
     * Retorna um faturamento específico
     * @param type $fatCodigo código do faturamento
     * @return type
     */
    public function getFaturamento($fatCodigo)
    {

        if (empty($fatCodigo)) {
            return Response::faillure("Informe o código do faturamento.");
        }

        $faturamento = Faturamento::with("os", "mvf", "vendas")->find($fatCodigo);

        if (!$faturamento) {
            return Response::faillure("O faturamento código {$fatCodigo} não foi encontrado.");
        }

        $faturamento->notasServico = [];
        $faturamento->notasItens = [];
        $faturamento->parcelas = [];

        //obtém os campos adicionais das classes associativas
        if ($faturamento->mvf) {
            $arrayParcelas = [];
            foreach ($faturamento->mvf as $key => $mvf) {
                $parcelas = Parcela::getParcelas([SCHModel::buildFilterObject("mvf_codigo", " = ", $mvf->fam_fk_mvf_codigo)]);
                foreach ($parcelas as $parcela) {
                    $arrayParcelas[] = $parcela;
                }
            }
            $faturamento->parcelas = $arrayParcelas;
        }

        //obtém os campos adicionais das classes associativas
        $notasServicos = [];
        $notasItens = [];
        $faturamentoNotas = FaturamentoNotaFiscal::where("fnf_fk_fat_codigo", $faturamento->fat_codigo)->get();
        if ($faturamentoNotas) {
            foreach ($faturamentoNotas as $fatNf) {
                if (!empty($fatNf->fnf_fk_nfs_codigo)) {
                    $notasServicos[] = NotaFiscalServico::where("nfs_codigo", $fatNf->fnf_fk_nfs_codigo)->with("filial", "tomador")->first();
                }
                if (!empty($fatNf->fnf_fk_nof_codigo)) {
                    $notasItens[] = NotaFiscal::getListaNotas([SCHModel::buildFilterObject("nof_codigo", " = ", $fatNf->fnf_fk_nof_codigo)]);
                }
            }
        }

        if (count($faturamento->vendas) > 0) {
            foreach ($faturamento->vendas as $fatVenda) {
                if ($fatVenda->venda->ven_fk_nof_codigo) {
                    $nf = NotaFiscal::getListaNotas([SCHModel::buildFilterObject("nof_codigo", " = ", $fatVenda->venda->ven_fk_nof_codigo)]);
                    $notasItens[] = $nf[0];
                }
            }
        }


        $faturamento->notasServico = $notasServicos;
        $faturamento->notasItens = $notasItens;

        return Response::success("", $faturamento);
    }

    public function getRelatorioProdutosOs($sto_codigo)
    {


        if (!is_numeric($sto_codigo)) {
            return Response::faillure("O filtro de Status é obrigatório");
        }


        $sql = "SELECT
               COALESCE(pve_codigo_auxiliar,pve_codigo::text)as codigo_produto
               , pro_descricao || ' ' || prv_descricao as nome_produto
               , unm_descricao as unidade_medida
               , sum(osps_quantidade) as quantidade
            FROM os.os
               INNER JOIN os.os_produtos_servicos on osps_fk_os_codigo = os_codigo
               INNER JOIN produto.produto_variedade_embalagem on pve_codigo = osps_fk_pve_codigo
               INNER JOIN produto.produto_variedade on prv_codigo = pve_fk_prv_codigo
               INNER JOIN produto.produto on pro_codigo = prv_fk_pro_codigo AND pro_servico = 0
               INNER JOIN cadastro.unidade_medida on unm_codigo = pve_fk_unm_codigo
            WHERE   os_fk_sto_codigo = ?
               GROUP BY pve_codigo, pve_codigo_auxiliar, pro_descricao, prv_descricao, pve_fk_unm_codigo, unm_descricao
               ORDER BY pro_descricao";
        $result = DB::select($sql, [$sto_codigo]);


        return Response::success("", $result);
    }

    public function getRelatorioProdutosOscomFiltro($sto_codigo, $osCampos)
    {
        $osCampos = \GuzzleHttp\json_decode($osCampos);
        if (!is_numeric($sto_codigo)) {
            return Response::faillure("O filtro de Status é obrigatório");
        }

        $sql = "SELECT
                 os_codigo
               , os_campos
               , COALESCE(pve_codigo_auxiliar,pve_codigo::text)as codigo_produto
               , pro_descricao || ' ' || prv_descricao as nome_produto
               , unm_descricao as unidade_medida
               , sum(osps_quantidade) as quantidade
            FROM os.os
               INNER JOIN os.os_produtos_servicos on osps_fk_os_codigo = os_codigo
               INNER JOIN produto.produto_variedade_embalagem on pve_codigo = osps_fk_pve_codigo
               INNER JOIN produto.produto_variedade on prv_codigo = pve_fk_prv_codigo
               INNER JOIN produto.produto on pro_codigo = prv_fk_pro_codigo AND pro_servico = 0
               INNER JOIN cadastro.unidade_medida on unm_codigo = pve_fk_unm_codigo
            WHERE   os_fk_sto_codigo = ?
               GROUP BY pve_codigo, pve_codigo_auxiliar, pro_descricao, prv_descricao, pve_fk_unm_codigo, unm_descricao, os_codigo
               ORDER BY pro_descricao";
        $result = DB::select($sql, [$sto_codigo]);

        $produtos = [];
        foreach ($result as $r) {
            $collection = collect(\GuzzleHttp\json_decode($r->os_campos), true);
            $c = $i = 0;
            foreach ($osCampos as $f) {
                if (isset($f[2][0])) {
                    $c = $collection->where($f[0][0], $f[0][1], $f[0][2])->where($f[1][0], $f[1][1], str_replace("-", "/", $f[1][2]))->count();
                } else {
                    $c = $collection->where($f[0][0], $f[0][1], $f[0][2])->where($f[1][0], $f[1][1], $f[1][2])->count();
                }
                if ($c != '0')
                    $i++;
            }
            if (count($osCampos) == $i)
                array_push($produtos, $r);
        }

        // Problema que estava dando: Produtos aparecendo duplicados no relat�rio
        // Optou-se por esse c�digo pelo fato de n�o ter como fazer um groupby nos campos json do banco de dados
        // Portanto, pegou-se o array j� tratado pela data e eliminou-se os produtos repetidos somando suas quantidades
        $value = json_decode(json_encode($produtos), true);
        $certo = $value;
        $ant = '';
        $index = 0;
        $cont = 0;
        foreach ($value as $valueCod) {
            $atual = $valueCod['codigo_produto'];
            if ($atual == $ant) {
                $cont++;
                array_splice($certo, $index - $cont + 1, 1);
                $certo[$index - $cont]['quantidade'] = $certo[$index - $cont]['quantidade'] + $valueCod['quantidade'];
            }
            $index++;
            $ant = $atual;
        }

        return Response::success("", $certo);
    }

    public function getRelatorioProdutosLocacao($filtro)
    {
        $filtro = \GuzzleHttp\json_decode($filtro);

        $sql = "SELECT
                  COALESCE(pve_codigo_auxiliar,pve_codigo::text)as codigo_produto
                , pro_descricao || ' ' || prv_descricao as nome_produto
                , unm_descricao as unidade_medida
                , sum(olo_quantidade) as quantidade
                , avg(olo_valor_unitario) as valor_unitario_medio
                , sum(olo_valor_unitario*olo_quantidade) as valor
                , sum(olo_valor_multa) as multa
             FROM os.os
            INNER JOIN os.os_locacao
               on olo_fk_os_codigo = os_codigo
            INNER JOIN produto.produto_variedade_embalagem
               on pve_codigo = olo_fk_pve_codigo
            INNER JOIN produto.produto_variedade
               on prv_codigo = pve_fk_prv_codigo
            INNER JOIN produto.produto
               on pro_codigo = prv_fk_pro_codigo AND pro_servico = 0
            INNER JOIN cadastro.unidade_medida
               on unm_codigo = pve_fk_unm_codigo

            WHERE 1 = 1 ";

        foreach ($filtro as $k => $f) {
            $op = " = ";
            if ($k == "olo_data_ini") {
                $op = " >= ";
            } else if ($k == "olo_data_fim") {
                $op = " <= ";
            }

            if ($k == "os_fk_sto_codigo" && !is_numeric($filtro->os_fk_sto_codigo)) {
                break;
            } else {
                $sql .= " AND ";
                $sql .= $k . $op . "'" . $f . "'" . " ";
            }
        }

        $sql .= "GROUP BY
                  pve_codigo
                , pve_codigo_auxiliar
                , pro_descricao
                , prv_descricao
                , pve_fk_unm_codigo
                , unm_descricao
            ORDER BY pro_descricao

               ";

        $result = DB::select($sql);

        return Response::success("", $result);
    }

    public function getRelatorioProdutosContrato($filtro)
    {

        $filtro = \GuzzleHttp\json_decode($filtro);

        $sql = "SELECT
                  olo_quantidade as quantidade,
                  olo_valor_unitario as valor,
                  olo_codigo as codigo,
                  olo_total_locacao as total,
                  sto_descricao as statusd
             FROM os.os_locacao
            INNER JOIN os.os
               on os_codigo = olo_fk_os_codigo
            INNER JOIN  os.status_os
               on sto_codigo = os_fk_sto_codigo

            WHERE 1 = 1";

        if ($filtro->codigo_pessoa != "")
            $sql .= "AND os_fk_pes_codigo = {$filtro->codigo_pessoa}";

        if ($filtro->codigo_produto != "")
            $sql .= "AND olo_fk_pve_codigo = {$filtro->codigo_produto}";

        $sql .= "ORDER BY olo_codigo";

        $result = DB::select($sql);

        return Response::success("", $result);
    }

    public function getRelatorioClientesLocacao($filtro)
    {

        $filtro = \GuzzleHttp\json_decode($filtro);

        $sql = "SELECT
                  pes_codigo as codigo_pessoa
                  , pes_nome as nome_pessoa
                  , unm_descricao as unidade_medida
                  , sum(olo_quantidade) as quantidade
                  , avg(olo_valor_unitario) as valor_unitario_medio
                  , sum(olo_valor_unitario*olo_quantidade) as valor
                  , sum(olo_valor_multa) as multa
             FROM cadastro.pessoa
            INNER JOIN os.os
               on os_fk_pes_codigo = pes_codigo
            INNER JOIN os.os_locacao
               on olo_fk_os_codigo = os_codigo
            INNER JOIN produto.produto_variedade_embalagem
               on pve_codigo = olo_fk_pve_codigo
            INNER JOIN produto.produto_variedade
               on prv_codigo = pve_fk_prv_codigo
            INNER JOIN produto.produto
               on pro_codigo = prv_fk_pro_codigo AND pro_servico = 0
            INNER JOIN cadastro.unidade_medida
               on unm_codigo = pve_fk_unm_codigo

            WHERE os_fk_pes_codigo = pes_codigo ";

        foreach ($filtro as $k => $f) {
            $op = " = ";
            if ($k == "olo_data_ini") {
                $op = " >= ";
            } else if ($k == "olo_data_fim") {
                $op = " <= ";
            }

            if ($k == "os_fk_sto_codigo" && !is_numeric($filtro->os_fk_sto_codigo)) {
                break;
            } else {
                $sql .= " AND ";
                $sql .= $k . $op . "'" . $f . "'" . " ";
            }
        }

        $sql .= "GROUP BY
               pes_codigo
               , pes_nome
               , pve_fk_unm_codigo
               , unm_descricao
               ORDER BY pes_nome";

        $result = DB::select($sql);

        return Response::success("", $result);
    }

    public static function buscarMvfsFaturamento($fatCodigo)
    {
        $fat = FaturamentoMvf::where('fam_fk_fat_codigo', $fatCodigo)->get();
        $mvfs = (object)Array();
        $k = 0;
        foreach ($fat as $fat) {
            $mvf = MovimentacaoFinanceira::find($fat->fam_fk_mvf_codigo);
            $mvfs->mvf[$k] = $mvf;
            $k++;
        }
        return Response::success("", $mvfs);
    }

    public static function getVendasRealizadas($osCodigo)
    {
        $osps = OsProdutosServicos::where("osps_fk_os_codigo", $osCodigo)->get();

        if (empty($osps)) {
            return Response::faillure("", "Nenhum registro encontrado");
        } else {
            return Response::success("", $osps);
        }
    }

    public function getVeiculos($placa = "")
    {
        if (!isset($placa) || empty($placa))
            return Response::success("", OsVeiculo::select("osv_placa", "osv_fabricante", "osv_modelo", "osv_ano", "osv_cor")
                ->groupBy("osv_placa", "osv_fabricante", "osv_modelo", "osv_ano", "osv_cor")
                ->get());
        return Response::success("", OsVeiculo::where("osv_placa", "ilike", "%" . $placa . "%")
            ->select("osv_placa", "osv_fabricante", "osv_modelo", "osv_ano", "osv_cor")
            ->groupBy("osv_placa", "osv_fabricante", "osv_modelo", "osv_ano", "osv_cor")
            ->get());
    }

    public static function consultarDocumentosEmail($osCodigo)
    {
        $faturamentoNotaFiscal = FaturamentoNotaFiscal::where('fnf_fk_os_codigo', $osCodigo)->whereNotNull('fnf_fk_nof_codigo')->get();
        $faturamentoNotaFiscalS = FaturamentoNotaFiscal::where('fnf_fk_os_codigo', $osCodigo)->whereNotNull('fnf_fk_nfs_codigo')->get();
        $faturamentoBoleto = FaturamentoOs::where('fao_fk_os_codigo', $osCodigo)->get();
        $objRetorno = (object)Array();

        if (!empty($faturamentoNotaFiscal) && sizeOf($faturamentoNotaFiscal) > 0) {
            $i = 0;
            foreach ($faturamentoNotaFiscal as $fnf) {
                $doc = Documento::where('doc_fk_nof_codigo', $fnf->fnf_fk_nof_codigo)->where('doc_status', 100)->first();
                if (!empty($doc)) {
                    $notaFiscal = NotaFiscal::find($fnf->fnf_fk_nof_codigo);
                    $objRetorno->nota[$i] = $notaFiscal;
                    $i++;
                }
            }
        }
        if (!empty($faturamentoNotaFiscalS) && sizeOf($faturamentoNotaFiscalS) > 0) {
            $i = 0;
            foreach ($faturamentoNotaFiscalS as $fnf) {
                $notaFiscalS = NotaFiscalServico::where('nfs_codigo', $fnf->fnf_fk_nfs_codigo)->whereNotNull('nfs_numero')->whereNull('nfs_data_cancelamento')->first();
                if (!empty($notaFiscalS)) {
                    $objRetorno->nfs[$i] = $notaFiscalS;
                    $i++;
                }
            }
        }
        if (!empty($faturamentoBoleto) && sizeOf($faturamentoBoleto) > 0) {
            $i = 0;
            foreach ($faturamentoBoleto as $fao) {
                $fatmvf = FaturamentoMvf::where('fam_fk_fat_codigo', $fao->fao_fk_fat_codigo)->first();
                if (!empty($fatmvf)) {
                    $mvf = MovimentacaoFinanceira::find($fatmvf->fam_fk_mvf_codigo);
                    $parcela = Parcela::where('par_fk_mvf_codigo', $mvf->mvf_codigo)->get();
                    foreach ($parcela as $par) {
                        $bolpar = BoletoParcela::where('bpar_fk_par_codigo', $par->par_codigo)->first();
                        if (!empty($bolpar)) {
                            $boleto = BoletoDetalhes::find($bolpar->bpar_fk_bde_codigo);
                            if (!empty($boleto)) {
                                $objRetorno->boleto[$i] = $boleto;
                                $i++;
                            }
                        }
                    }
                }
            }
        }
        return Response::success("", $objRetorno);
    }

    public static function consultarSerialOrcamento($pve_codigo, $serial, $local_estoque)
    {
        $snap = Snapshot::where('sna_fk_pve_codigo', $pve_codigo)->where('sna_fk_loe_codigo', $local_estoque)->where('sna_serial', $serial)->first();
        if (!empty($snap)) {
            $osps = OsProdutosServicos::where('osps_fk_pve_codigo', $pve_codigo)->where('osps_serial', $serial)->first();
            if (!empty($osps)) {
                $possuiOrcamentoSerial = $osps->osps_fk_os_codigo;
            } else {
                $possuiOrcamentoSerial = 0;
            }
        }
        return Response::success("", $possuiOrcamentoSerial);
    }

    /**
     * Retorna os históricos de uma OS, podendo ser filtrado por tipo de histórico
     *
     * @param type $codigoOs
     * @param type $tipoHistorico
     * @return type
     */
    public function getHistorico($codigoOs, $tipoHistorico = '')
    {

        $os = Os::find($codigoOs);
        if (!$os) {
            Response::faillure("A OS/Orçamento código {$codigoOs} não foi encontrada.");
        }

        $logOs = OsLog::where("osl_fk_os_codigo", $codigoOs);
        if (!empty($tipoHistorico)) {
            $logOs = $logOs->where("osl_fk_too_codigo", $tipoHistorico);
        }
        $logOs = $logOs->orderBy("osl_date_insert", "desc")->get();
        return Response::success("", $logOs);
    }

    public function last_os_numero_tipo($os_tipo = null){
        if($os_tipo){
            return ServiceOrder::orderBy('os_numero_tipo', 'DESC')->where('os_tipo',$os_tipo)->whereNotNull('os_numero_tipo')->first('os_numero_tipo')->os_numero_tipo ?? 0;
        }
        return ServiceOrder::orderBy('os_numero_tipo', 'DESC')->whereNotNull('os_numero_tipo')->first('os_numero_tipo')->os_numero_tipo ?? 0;
    }


}
